/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;

/**
 * 
 */
public class MultiConnTCPChannelOutImpl extends AbstractTCPChannelOutImpl
    implements
      MultiConnTCPChannelOutAttributeController
{

  protected String        chunkName       = IPChannelDestinationChunk.DEFAULT_NAME;

  protected Map           openConnections = new HashMap();
  protected ConnectionKey reusableKey     = new ConnectionKey();

  protected int           maxConnection   = 0;

  protected int           soTimeout       = -1;

  /**
   * @see org.objectweb.dream.Push#push(Message, Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    IPChannelDestinationChunk destination = (IPChannelDestinationChunk) message
        .getChunk(chunkName);
    // try to remove chunk
    if (message instanceof ExtensibleMessage)
    {
      messageManagerItf.deleteChunk(((ExtensibleMessage) message)
          .removeChunk(chunkName));
    }

  }

  // ---------------------------------------------------------------------------
  // Utility method
  // ---------------------------------------------------------------------------

  /*protected ConnectionState getConnection(IPChannelDestinationChunk destination)
  {
    synchronized (openConnections)
    {
      reusableKey.inetAddress = destination.getChannelDestinationAddr();
      reusableKey.port = destination.getChannelDestinationPort();
      ConnectionState connectionState = (ConnectionState) openConnections
          .get(reusableKey);
      if (connectionState == null)
      {
        
      }
    }
  }*/

  // ---------------------------------------------------------------------------
  // Overriden AbstractTCPChannelOutImpl method
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractTCPChannelOutImpl#setSocketOption(Socket)
   */
  protected void setSocketOption(Socket sock) throws SocketException
  {
    if (soTimeout >= 0)
    {
      sock.setSoTimeout(soTimeout);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the MultiDestinationChannelOutAttributeController
  // interface
  // ---------------------------------------------------------------------------

  /**
   * @see MultiDestinationChannelOutAttributeController#getDestinationChunkName()
   */
  public String getDestinationChunkName()
  {
    return chunkName;
  }

  /**
   * @see MultiDestinationChannelOutAttributeController#setDestinationChunkName(String)
   */
  public void setDestinationChunkName(String name)
  {
    chunkName = name;
  }

  // ---------------------------------------------------------------------------
  // Implementation of MultiConnTCPChannelInAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see MultiConnTCPChannelInAttributeController#getMaxConnection()
   */
  public int getMaxConnection()
  {
    return maxConnection;
  }

  /**
   * @see MultiConnTCPChannelInAttributeController#setMaxConnection(int)
   */
  public void setMaxConnection(int maxConnection)
  {
    this.maxConnection = maxConnection;
    // TODO remove connections.
  }

  /**
   * @see MultiConnTCPChannelInAttributeController#getSoTimeout()
   */
  public int getSoTimeout()
  {
    return soTimeout;
  }

  /**
   * @see MultiConnTCPChannelInAttributeController#setSoTimeout(int)
   */
  public void setSoTimeout(int timeout)
  {
    soTimeout = timeout;
  }

  // ---------------------------------------------------------------------------
  // Utility inner classes
  // ---------------------------------------------------------------------------

  protected class ConnectionKey
  {

    protected InetAddress inetAddress;
    protected int         port;

    /**
     * @see Object#equals(Object)
     */
    public boolean equals(Object obj)
    {
      final ConnectionKey key = (ConnectionKey) obj;
      return (inetAddress == null ? key.inetAddress == null : inetAddress
          .equals(key.inetAddress))
          && port == key.port;
    }

    /**
     * @see Object#hashCode()
     */
    public int hashCode()
    {
      return inetAddress == null ? inetAddress.hashCode() + port : port;
    }
  }
}